#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import mock

from congressclient.osc.v1 import driver
from congressclient.tests import common


class TestListDrivers(common.TestCongressBase):

    def test_list_drivers(self):
        arglist = [
        ]
        verifylist = [
        ]
        response = {
            "results": [{"id": "neutronv2",
                         "description": "this does blah.."}]
        }
        lister = mock.Mock(return_value=response)
        self.app.client_manager.congressclient.list_drivers = lister
        cmd = driver.ListDrivers(self.app, self.namespace)

        parsed_args = self.check_parser(cmd, arglist, verifylist)
        result = cmd.take_action(parsed_args)

        lister.assert_called_with()
        self.assertEqual(['id', 'description'], result[0])


class TestShowDriverSchema(common.TestCongressBase):

    def test_show_driver_shema(self):
        arglist = [
            "neutronv2"
        ]
        verifylist = [
            ('driver', "neutronv2")
        ]

        response = {
            "tables":
                [{'table_id': 'ports',
                  'columns': [{"name": "name", "description": "None"},
                              {"name": "status", "description": "None"},
                              {"name": "id", "description": "None"}]},
                 {'table_id': 'routers',
                  'columns': [{"name": "name", "description": "None"},
                              {"name": "floating_ip", "description": "None"},
                              {"name": "id", "description": "None"}]}]
        }
        lister = mock.Mock(return_value=response)
        self.app.client_manager.congressclient.show_driver = lister
        cmd = driver.ShowDriverSchema(self.app, self.namespace)

        parsed_args = self.check_parser(cmd, arglist, verifylist)
        result = cmd.take_action(parsed_args)

        lister.assert_called_with("neutronv2")
        self.assertEqual(['table', 'columns'], result[0])


class TestShowDriverConfig(common.TestCongressBase):

    def test_show_driver_config(self):
        arglist = [
            "neutronv2"
        ]
        verifylist = [
            ('driver', "neutronv2")
        ]

        response = {
            "tables": [],
            'id': 'aabbcc',
            'description': 'foobar',
            'config': {'password': 'password'},
        }
        mocker = mock.Mock(return_value=response)
        self.app.client_manager.congressclient.show_driver = mocker
        cmd = driver.ShowDriverConfig(self.app, self.namespace)

        parsed_args = self.check_parser(cmd, arglist, verifylist)
        result = list(cmd.take_action(parsed_args))

        mocker.assert_called_with("neutronv2")
        filtered = [('config', 'description', 'id'),
                    (response['config'], response['description'],
                     response['id'])]
        self.assertEqual(filtered, result)



#   Copyright 2012-2013 OpenStack Foundation
#   Copyright 2013 Nebula Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import os
import sys

import fixtures
import testtools

from congressclient.tests import fakes


class TestCase(testtools.TestCase):
    def setUp(self):
        testtools.TestCase.setUp(self)

        if (os.environ.get("OS_STDOUT_CAPTURE") == "True" or
                os.environ.get("OS_STDOUT_CAPTURE") == "1"):
            stdout = self.useFixture(fixtures.StringStream("stdout")).stream
            self.useFixture(fixtures.MonkeyPatch("sys.stdout", stdout))

        if (os.environ.get("OS_STDERR_CAPTURE") == "True" or
                os.environ.get("OS_STDERR_CAPTURE") == "1"):
            stderr = self.useFixture(fixtures.StringStream("stderr")).stream
            self.useFixture(fixtures.MonkeyPatch("sys.stderr", stderr))

    def assertNotCalled(self, m, msg=None):
        """Assert a function was not called."""

        if m.called:
            if not msg:
                msg = 'method %s should not have been called' % m
            self.fail(msg)

    # 2.6 doesn't have the assert dict equals so make sure that it exists
    if tuple(sys.version_info)[0:2] < (2, 7):

        def assertIsInstance(self, obj, cls, msg=None):
            """self.assertTrue(isinstance(obj, cls)), with a nicer message."""

            if not isinstance(obj, cls):
                standardMsg = '%s is not an instance of %r' % (obj, cls)
                self.fail(self._formatMessage(msg, standardMsg))

        def assertDictEqual(self, d1, d2, msg=None):
            # Simple version taken from 2.7
            self.assertIsInstance(d1, dict,
                                  'First argument is not a dictionary')
            self.assertIsInstance(d2, dict,
                                  'Second argument is not a dictionary')
            if d1 != d2:
                if msg:
                    self.fail(msg)
                else:
                    standardMsg = '%r != %r' % (d1, d2)
                    self.fail(standardMsg)


class TestCommand(TestCase):
    """Test cliff command classes."""

    def setUp(self):
        super(TestCommand, self).setUp()
        # Build up a fake app
        self.fake_stdout = fakes.FakeStdout()
        self.app = fakes.FakeApp(self.fake_stdout)
        self.app.client_manager = fakes.FakeClientManager()

    def check_parser(self, cmd, args, verify_args):
        cmd_parser = cmd.get_parser('check_parser')
        try:
            parsed_args = cmd_parser.parse_args(args)
        except SystemExit:
            raise Exception("Argument parse failed")
        for av in verify_args:
            attr, value = av
            if attr:
                self.assertIn(attr, parsed_args)
                self.assertEqual(getattr(parsed_args, attr), value)
        return parsed_args



#   Copyright 2012-2013 OpenStack, LLC.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

"""Policy action implemenations"""

import sys

from cliff import command
from cliff import lister
from cliff import show
from keystoneauth1 import exceptions
from oslo_log import log as logging
from oslo_serialization import jsonutils
import six
import yaml

from congressclient.common import utils


def _format_rule(rule):
    """Break up rule string so it fits on screen."""

    rule_split = jsonutils.dumps(rule).split(":-")
    formatted_string = rule_split[0] + ":-\n"
    for rule in rule_split[1].split("), "):
        formatted_string += rule + '\n'
    return formatted_string


def get_rule_id_from_name(client, parsed_args):
    results = client.list_policy_rules(parsed_args.policy_name)['results']
    rule_id = None
    for result in results:
        if result.get('name') == parsed_args.rule_id:
            if rule_id is None:
                rule_id = result.get('id')
            else:
                raise exceptions.Conflict(
                    "[Multiple rules with same name: %s]" %
                    parsed_args.rule_id)
    if rule_id is None:
        raise exceptions.NotFound(
            "[No rule found with name: %s]" % parsed_args.rule_id)
    return rule_id


class CreatePolicyRule(show.ShowOne):
    """Create a policy rule."""

    log = logging.getLogger(__name__ + '.CreatePolicyRule')

    def get_parser(self, prog_name):
        parser = super(CreatePolicyRule, self).get_parser(prog_name)
        parser.add_argument(
            'policy_name',
            metavar="<policy-name>",
            help="Name or identifier of the policy")
        parser.add_argument(
            'rule',
            metavar="<rule>",
            help="Policy rule")
        parser.add_argument(
            '--name', dest="rule_name",
            help="Name of the policy rule")
        parser.add_argument(
            '--comment', dest="comment",
            help="Comment about policy rule")
        return parser

    def take_action(self, parsed_args):
        self.log.debug('take_action(%s)' % parsed_args)
        # set default max-width
        if parsed_args.max_width == 0:
            parsed_args.max_width = 80
        client = self.app.client_manager.congressclient
        body = {'rule': parsed_args.rule}
        if parsed_args.rule_name:
            body['name'] = parsed_args.rule_name
        if parsed_args.comment:
            body['comment'] = parsed_args.comment
        data = client.create_policy_rule(parsed_args.policy_name, body)
        return zip(*sorted(six.iteritems(data)))


class DeletePolicyRule(command.Command):
    """Delete a policy rule."""

    log = logging.getLogger(__name__ + '.DeletePolicyRule')

    def get_parser(self, prog_name):
        parser = super(DeletePolicyRule, self).get_parser(prog_name)
        parser.add_argument(
            'policy_name',
            metavar="<policy-name>",
            help="Name of the policy to delete")
        parser.add_argument(
            'rule_id',
            metavar="<rule-id/rule-name>",
            help="ID/Name of the policy rule to delete")
        return parser

    def take_action(self, parsed_args):
        self.log.debug('take_action(%s)' % parsed_args)
        client = self.app.client_manager.congressclient
        results = client.list_policy_rules(parsed_args.policy_name)
        rule_id = utils.get_resource_id_from_name(
            parsed_args.rule_id, results)
        client.delete_policy_rule(parsed_args.policy_name, rule_id)


class ListPolicyRules(command.Command):
    """List policy rules."""

    log = logging.getLogger(__name__ + '.ListPolicyRules')

    def get_parser(self, prog_name):
        parser = super(ListPolicyRules, self).get_parser(prog_name)
        parser.add_argument(
            'policy_name',
            metavar="<policy-name>",
            help="Name of the policy")
        return parser

    def take_action(self, parsed_args):
        self.log.debug('take_action(%s)' % parsed_args)
        client = self.app.client_manager.congressclient
        results = client.list_policy_rules(parsed_args.policy_name)['results']
        for result in results:
            print("// ID: %s" % str(result['id']))
            print("// Name: %s" % str(result.get('name')))
            if result['comment'] != "None" and result['comment']:
                print("// %s" % str(result['comment']))
            print(result['rule'])
            print('')
        return 0


class SimulatePolicy(command.Command):
    """Show the result of simulation."""

    log = logging.getLogger(__name__ + '.SimulatePolicy')

    def get_parser(self, prog_name):
        parser = super(SimulatePolicy, self).get_parser(prog_name)
        parser.add_argument(
            'policy',
            metavar="<policy>",
            help="Name of the policy")
        parser.add_argument(
            'query',
            metavar="<query>",
            help="String representing query (policy rule or literal)")
        parser.add_argument(
            'sequence',
            metavar="<sequence>",
            help="String representing sequence of updates/actions")
        parser.add_argument(
            'action_policy',
            metavar="<action_policy>",
            help="Name of the policy with actions",
            default=None)
        parser.add_argument(
            '--delta',
            action='store_true',
            default=False,
            help="Return difference in query caused by update sequence")
        parser.add_argument(
            '--trace',
            action='store_true',
            default=False,
            help="Include trace describing computation")
        return parser

    def take_action(self, parsed_args):
        self.log.debug('take_action(%s)' % parsed_args)
        client = self.app.client_manager.congressclient
        args = {}
        args['query'] = parsed_args.query
        args['sequence'] = parsed_args.sequence
        if parsed_args.action_policy is not None:
            args['action_policy'] = parsed_args.action_policy
        if parsed_args.delta:
            args['delta'] = parsed_args.delta
        if parsed_args.trace:
            args['trace'] = parsed_args.trace

        body = {'query': parsed_args.query,
                'sequence': parsed_args.sequence,
                'action_policy': parsed_args.action_policy}

        results = client.execute_policy_action(
            policy_name=parsed_args.policy,
            action="simulate",
            trace=parsed_args.trace,
            delta=parsed_args.delta,
            body=body)
        for result in results['result']:
            print(result)
        if 'trace' in results:
            print(results['trace'])
        return 0


class ListPolicyTables(lister.Lister):
    """List policy tables."""

    log = logging.getLogger(__name__ + '.ListPolicyTables')

    def get_parser(self, prog_name):
        parser = super(ListPolicyTables, self).get_parser(prog_name)
        parser.add_argument(
            'policy_name',
            metavar="<policy-name>",
            help="Name of the policy")
        return parser

    def take_action(self, parsed_args):
        self.log.debug('take_action(%s)' % parsed_args)
        # set default max-width
        if parsed_args.max_width == 0:
            parsed_args.max_width = 80
        client = self.app.client_manager.congressclient
        data = client.list_policy_tables(parsed_args.policy_name)['results']
        columns = ['id']
        formatters = {'PolicyTables': utils.format_list}
        return (columns,
                (utils.get_dict_properties(s, columns,
                                           formatters=formatters)
                 for s in data))


class ListPolicy(lister.Lister):
    """List Policy."""

    log = logging.getLogger(__name__ + '.ListPolicy')

    def get_parser(self, prog_name):
        parser = super(ListPolicy, self).get_parser(prog_name)
        return parser

    def take_action(self, parsed_args):
        # set default max-width
        if parsed_args.max_width == 0:
            parsed_args.max_width = 80
        client = self.app.client_manager.congressclient
        data = client.list_policy()['results']
        columns = ['id', 'name', 'owner_id', 'kind', 'description']
        formatters = {'Policies': utils.format_list}
        return (columns,
                (utils.get_dict_properties(s, columns,
                                           formatters=formatters)
                 for s in data))


class CreatePolicy(show.ShowOne):
    """Create a policy."""

    log = logging.getLogger(__name__ + '.CreatePolicy')

    def get_parser(self, prog_name):
        parser = super(CreatePolicy, self).get_parser(prog_name)
        parser.add_argument(
            'policy_name',
            metavar="<policy_name>",
            help="Name of the policy")
        parser.add_argument(
            '--description',
            metavar="<description>",
            help="Policy description")
        parser.add_argument(
            '--abbreviation',
            metavar="<abbreviation>",
            help="Policy abbreviation (used in traces). The length of the "
                 "string must be equal to or less than 5 characters. Defaults "
                 "to the first five characters of policy_name if not set.")
        parser.add_argument(
            '--kind',
            metavar="<kind>",
            choices=['nonrecursive', 'database', 'action', 'materialized',
                     'z3'],
            help="Kind of policy: "
                 "{nonrecursive, database, action, materialized, z3}")
        return parser

    def take_action(self, parsed_args):
        self.log.debug('take_action(%s)' % parsed_args)
        # set default max-width
        if parsed_args.max_width == 0:
            parsed_args.max_width = 80
        client = self.app.client_manager.congressclient
        body = {'name': parsed_args.policy_name,
                'description': parsed_args.description,
                'abbreviation': parsed_args.abbreviation,
                'kind': parsed_args.kind}
        data = client.create_policy(body)
        return zip(*sorted(six.iteritems(data)))


class CreatePolicyFromFile(show.ShowOne):
    """Create a policy."""

    log = logging.getLogger(__name__ + '.CreatePolicy')

    def get_parser(self, prog_name):
        parser = super(CreatePolicyFromFile, self).get_parser(prog_name)
        parser.add_argument(
            'policy_file_path',
            metavar="<policy_file_path>",
            help="Path to policy file")
        return parser

    def take_action(self, parsed_args):
        self.log.debug('take_action(%s)' % parsed_args)
        # set default max-width
        if parsed_args.max_width == 0:
            parsed_args.max_width = 80
        client = self.app.client_manager.congressclient
        with open(parsed_args.policy_file_path, "r") as stream:
            policies = yaml.load_all(stream)
            try:
                body = next(policies)
            except StopIteration:
                raise Exception('No policy found in file.')
            try:
                body = next(policies)
                raise Exception(
                    'More than one policy found in file. None imported.')
            except StopIteration:
                pass
        data = client.create_policy(body)

        def rule_dict_to_string(rules):
            rule_str_list = [rule['rule'] for rule in rules]
            return "\n".join(rule_str_list)

        data['rules'] = rule_dict_to_string(data['rules'])
        return zip(*sorted(six.iteritems(data)))


class DeletePolicy(command.Command):
    """Delete a policy."""

    log = logging.getLogger(__name__ + '.DeletePolicy')

    def get_parser(self, prog_name):
        parser = super(DeletePolicy, self).get_parser(prog_name)
        parser.add_argument(
            'policy',
            metavar="<policy>",
            help="ID or name of the policy to delete")
        return parser

    def take_action(self, parsed_args):
        self.log.debug('take_action(%s)' % parsed_args)
        client = self.app.client_manager.congressclient

        client.delete_policy(parsed_args.policy)


class ListPolicyRows(lister.Lister):
    """List policy rows."""

    log = logging.getLogger(__name__ + '.ListPolicyRows')

    def get_parser(self, prog_name):
        parser = super(ListPolicyRows, self).get_parser(prog_name)
        parser.add_argument(
            'policy_name',
            metavar="<policy-name>",
            help="Name of the policy to show")
        parser.add_argument(
            'table',
            metavar="<table>",
            help="Table to get the policy rows from")
        parser.add_argument(
            '--trace',
            action='store_true',
            default=False,
            help="Display explanation of result")
        return parser

    def take_action(self, parsed_args):
        self.log.debug('take_action(%s)' % parsed_args)
        # set default max-width
        if parsed_args.max_width == 0:
            parsed_args.max_width = 80
        client = self.app.client_manager.congressclient
        answer = client.list_policy_rows(parsed_args.policy_name,
                                         parsed_args.table,
                                         parsed_args.trace)

        if 'trace' in answer:
            sys.stdout.write(answer['trace'] + '\n')
        results = answer['results']
        columns = []
        if results:
            columns = ['Col%s' % (i)
                       for i in range(0, len(results[0]['data']))]
        self.log.debug("Columns: " + str(columns))
        return (columns, (x['data'] for x in results))


class ShowPolicyRule(show.ShowOne):
    """Show a policy rule."""

    log = logging.getLogger(__name__ + '.ShowPolicyRule')

    def get_parser(self, prog_name):
        parser = super(ShowPolicyRule, self).get_parser(prog_name)
        parser.add_argument(
            'policy_name',
            metavar="<policy-name>",
            help="Name or identifier of the policy")
        parser.add_argument(
            'rule_id',
            metavar="<rule-id/rule-name>",
            help="Policy rule id or rule name")

        return parser

    def take_action(self, parsed_args):
        self.log.debug('take_action(%s)' % parsed_args)
        # set default max-width
        if parsed_args.max_width == 0:
            parsed_args.max_width = 80
        client = self.app.client_manager.congressclient
        results = client.list_policy_rules(parsed_args.policy_name)
        rule_id = utils.get_resource_id_from_name(
            parsed_args.rule_id, results)
        data = client.show_policy_rule(parsed_args.policy_name, rule_id)
        return zip(*sorted(six.iteritems(data)))


class ShowPolicyTable(show.ShowOne):
    """Show policy table properties."""

    log = logging.getLogger(__name__ + '.ShowPolicyTable')

    def get_parser(self, prog_name):
        parser = super(ShowPolicyTable, self).get_parser(prog_name)
        parser.add_argument(
            'policy_name',
            metavar='<policy-name>',
            help="Name of policy")
        parser.add_argument(
            'table_id',
            metavar='<table-id>',
            help="Table id")

        return parser

    def take_action(self, parsed_args):
        self.log.debug('take_action(%s)' % parsed_args)
        # set default max-width
        if parsed_args.max_width == 0:
            parsed_args.max_width = 80
        client = self.app.client_manager.congressclient
        data = client.show_policy_table(parsed_args.policy_name,
                                        parsed_args.table_id)
        return zip(*sorted(six.iteritems(data)))


class ShowPolicy(show.ShowOne):
    """Show policy properties."""

    log = logging.getLogger(__name__ + '.ShowPolicy')

    def get_parser(self, prog_name):
        parser = super(ShowPolicy, self).get_parser(prog_name)
        parser.add_argument(
            'policy_name',
            metavar='<policy-name>',
            help="Name of policy")

        return parser

    def take_action(self, parsed_args):
        self.log.debug('take_action(%s)' % parsed_args)
        # set default max-width
        if parsed_args.max_width == 0:
            parsed_args.max_width = 80
        client = self.app.client_manager.congressclient
        results = client.list_policy()
        policy_id = utils.get_resource_id_from_name(
            parsed_args.policy_name, results)
        data = client.show_policy(policy_id)
        return zip(*sorted(six.iteritems(data)))



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
import os

import mock

from congressclient.common import utils
from congressclient.osc.v1 import policy
from congressclient.tests import common


class TestCreatePolicy(common.TestCongressBase):

    def test_create_policy(self):
        policy_name = 'test1'
        policy_id = "e531f2b3-3d97-42c0-b3b5-b7b6ab532018"
        response = {"description": "",
                    "id": policy_id,
                    "name": policy_name,
                    "kind": "nonrecursive",
                    "owner": "system",
                    "abbreviation": "test1"}

        arglist = [policy_name]
        verifylist = [
            ('policy_name', policy_name),
        ]

        mocker = mock.Mock(return_value=response)
        self.app.client_manager.congressclient.create_policy = mocker
        cmd = policy.CreatePolicy(self.app, self.namespace)
        parsed_args = self.check_parser(cmd, arglist, verifylist)
        result = list(cmd.take_action(parsed_args))
        filtered = [('abbreviation', 'description', 'id', 'kind', 'name',
                     'owner'),
                    (policy_name, '', policy_id, 'nonrecursive',
                     policy_name, 'system')]
        self.assertEqual(filtered, result)


class TestCreatePolicyFromFile(common.TestCongressBase):

    def test_create_policy(self):
        policy_path = os.path.dirname(
            os.path.abspath(__file__)) + '/test_policy_file.yaml'
        policy_id = "e531f2b3-3d97-42c0-b3b5-b7b6ab532018"
        response = {"description": "",
                    "id": policy_id,
                    "name": "test_policy",
                    "kind": "nonrecursive",
                    "owner": "system",
                    "abbreviation": "abbr",
                    "rules": [
                        {'comment': 'test comment', 'name': 'test name',
                         'rule': 'p(x) :-  q(x)'},
                        {'comment': 'test comment2', 'name': 'test name2',
                         'rule': 'p(x) :- q2(x)'}]}

        arglist = [policy_path]
        verifylist = [
            ('policy_file_path', policy_path),
        ]

        mocker = mock.Mock(return_value=response)
        self.app.client_manager.congressclient.create_policy = mocker
        cmd = policy.CreatePolicyFromFile(self.app, self.namespace)
        parsed_args = self.check_parser(cmd, arglist, verifylist)
        result = list(cmd.take_action(parsed_args))
        filtered = [('abbreviation', 'description', 'id', 'kind', 'name',
                     'owner', 'rules'),
                    ('abbr', '', policy_id, 'nonrecursive',
                     'test_policy', 'system',
                     'p(x) :-  q(x)\n'
                     'p(x) :- q2(x)')]
        self.assertEqual(filtered, result)


class TestShowPolicy(common.TestCongressBase):
    def test_show_policy(self):
        policy_id = "14f2897a-155a-4c9d-b3de-ef85c0a171d8"
        policy_name = "test1"
        arglist = [policy_id]
        verifylist = [
            ('policy_name', policy_id),
        ]
        response = {"description": "",
                    "id": policy_id,
                    "name": policy_name,
                    "kind": "nonrecursive",
                    "owner": "system",
                    "abbreviation": "test1"}

        mocker = mock.Mock(return_value=response)
        self.app.client_manager.congressclient.show_policy = mocker
        self.app.client_manager.congressclient.list_policy = mock.Mock()
        cmd = policy.ShowPolicy(self.app, self.namespace)
        parsed_args = self.check_parser(cmd, arglist, verifylist)
        with mock.patch.object(utils, "get_resource_id_from_name",
                               return_value="name"):
            result = list(cmd.take_action(parsed_args))
        filtered = [('abbreviation', 'description', 'id', 'kind', 'name',
                     'owner'),
                    (policy_name, '', policy_id, 'nonrecursive',
                     policy_name, 'system')]
        self.assertEqual(filtered, result)


class TestDeletePolicy(common.TestCongressBase):
    def test_delete_policy(self):
        policy_id = 'e531f2b3-3d97-42c0-b3b5-b7b6ab532018'
        arglist = [
            policy_id
        ]
        verifylist = [
            ('policy', policy_id)
        ]
        mocker = mock.Mock(return_value=None)
        self.app.client_manager.congressclient.delete_policy = mocker
        self.app.client_manager.congressclient.list_policy = mock.Mock()
        cmd = policy.DeletePolicy(self.app, self.namespace)

        parsed_args = self.check_parser(cmd, arglist, verifylist)
        with mock.patch.object(utils, "get_resource_id_from_name",
                               return_value="id"):
            result = cmd.take_action(parsed_args)

        mocker.assert_called_with(policy_id)
        self.assertIsNone(result)


class TestCreatePolicyRule(common.TestCongressBase):

    def test_create_policy_rule(self):
        policy_name = 'classification'
        rule = "p(x) :- q(x)"
        response = {"comment": "Comment",
                    "id": "e531f2b3-3d97-42c0-b3b5-b7b6ab532018",
                    "rule": rule}

        arglist = [policy_name, rule]
        verifylist = [
            ('policy_name', policy_name),
            ('rule', rule),
        ]

        mocker = mock.Mock(return_value=response)
        self.app.client_manager.congressclient.create_policy_rule = mocker
        cmd = policy.CreatePolicyRule(self.app, self.namespace)
        parsed_args = self.check_parser(cmd, arglist, verifylist)
        result = list(cmd.take_action(parsed_args))
        filtered = [('comment', 'id', 'rule'),
                    ('Comment', 'e531f2b3-3d97-42c0-b3b5-b7b6ab532018', rule)]
        self.assertEqual(filtered, result)

    def test_create_policy_rule_with_name(self):
        policy_name = "classification"
        rule = "p(x) :- q(x)"
        rule_name = "classification_rule"
        response = {"comment": "None",
                    "id": "e531f2b3-3d97-42c0-b3b5-b7b6ab532018",
                    "rule": rule,
                    "name": rule_name}

        arglist = ["--name", rule_name, policy_name, rule]
        verifylist = [
            ('policy_name', policy_name),
            ('rule', rule),
            ("rule_name", rule_name)
        ]

        mocker = mock.Mock(return_value=response)
        self.app.client_manager.congressclient.create_policy_rule = mocker
        cmd = policy.CreatePolicyRule(self.app, self.namespace)
        parsed_args = self.check_parser(cmd, arglist, verifylist)
        result = list(cmd.take_action(parsed_args))
        filtered = [('comment', 'id', 'name', 'rule'),
                    ('None', 'e531f2b3-3d97-42c0-b3b5-b7b6ab532018', rule_name,
                     rule)]
        self.assertEqual(filtered, result)


class TestDeletePolicyRule(common.TestCongressBase):
    def test_delete_policy_rule(self):
        policy_name = 'classification'
        rule_id = 'e531f2b3-3d97-42c0-b3b5-b7b6ab532018'
        arglist = [
            policy_name, rule_id
        ]
        verifylist = [
            ('policy_name', policy_name),
            ('rule_id', rule_id)
        ]
        mocker = mock.Mock(return_value=None)
        self.app.client_manager.congressclient.delete_policy_rule = mocker
        self.app.client_manager.congressclient.list_policy_rules = mock.Mock()
        cmd = policy.DeletePolicyRule(self.app, self.namespace)

        parsed_args = self.check_parser(cmd, arglist, verifylist)
        with mock.patch.object(utils, "get_resource_id_from_name",
                               return_value=rule_id):
            result = cmd.take_action(parsed_args)

        mocker.assert_called_with(policy_name, rule_id)
        self.assertIsNone(result)


class TestListPolicyRules(common.TestCongressBase):
    def test_list_policy_rules(self):
        policy_name = 'classification'
        rule_id = 'e531f2b3-3d97-42c0-b3b5-b7b6ab532018'
        arglist = [
            policy_name
        ]
        verifylist = [
            ('policy_name', policy_name)
        ]
        response = {
            "results": [{"comment": "None",
                         "id": rule_id,
                         "rule": "security_group(port, security_group_name)"
                         }]
        }
        lister = mock.Mock(return_value=response)
        self.app.client_manager.congressclient.list_policy_rules = lister
        cmd = policy.ListPolicyRules(self.app, self.namespace)

        parsed_args = self.check_parser(cmd, arglist, verifylist)
        cmd.take_action(parsed_args)

        lister.assert_called_with(policy_name)


class TestListPolicy(common.TestCongressBase):
    def test_list_policy_rules(self):
        policy_name = 'classification'
        policy_id = 'e531f2b3-3d97-42c0-b3b5-b7b6ab532018'
        arglist = [
        ]
        verifylist = [
        ]
        response = {
            "results": [{"id": policy_id,
                         "owner_id": "system",
                         "name": policy_name,
                         "kind": "nonrecursive",
                         "description": "my description"
                         }]}
        lister = mock.Mock(return_value=response)
        self.app.client_manager.congressclient.list_policy = lister
        cmd = policy.ListPolicy(self.app, self.namespace)

        parsed_args = self.check_parser(cmd, arglist, verifylist)
        result = cmd.take_action(parsed_args)

        lister.assert_called_with()
        self.assertEqual(['id', 'name', 'owner_id', 'kind', 'description'],
                         result[0])


class TestListPolicyTables(common.TestCongressBase):
    def test_list_policy_tables(self):
        policy_name = 'classification'
        arglist = [
            policy_name
        ]
        verifylist = [
            ('policy_name', policy_name)
        ]
        response = {
            "results": [{"id": "ports"},
                        {"id": "virtual_machines"}]
        }
        lister = mock.Mock(return_value=response)
        self.app.client_manager.congressclient.list_policy_tables = lister
        cmd = policy.ListPolicyTables(self.app, self.namespace)

        parsed_args = self.check_parser(cmd, arglist, verifylist)
        result = cmd.take_action(parsed_args)

        lister.assert_called_with(policy_name)
        self.assertEqual(['id'], result[0])


class TestListPolicyRows(common.TestCongressBase):

    def test_list_policy_rules(self):
        policy_name = 'classification'
        table_name = 'port_security_group'
        arglist = [
            policy_name, table_name
        ]
        verifylist = [
            ('policy_name', policy_name),
            ('table', table_name)
        ]
        response = {"results":
                    [{"data": ["69abc88b-c950-4625-801b-542e84381509",
                               "default"]}]}

        lister = mock.Mock(return_value=response)
        self.app.client_manager.congressclient.list_policy_rows = lister
        cmd = policy.ListPolicyRows(self.app, self.namespace)

        parsed_args = self.check_parser(cmd, arglist, verifylist)
        cmd.take_action(parsed_args)

        lister.assert_called_with(policy_name, table_name, False)

    def test_list_policy_rules_trace(self):
        policy_name = 'classification'
        table_name = 'p'
        arglist = [
            policy_name, table_name, "--trace"
        ]
        verifylist = [
            ('policy_name', policy_name),
            ('table', table_name)
        ]
        response = {"results":
                    [{"data": ["69abc88b-c950-4625-801b-542e84381509",
                               "default"]}],
                    "trace": "Call p(x, y)\n "
                             "Exit p('69abc88b-c950-4625-801b-542e84381509', "
                             "'default')\n"}

        lister = mock.Mock(return_value=response)
        self.app.client_manager.congressclient.list_policy_rows = lister
        cmd = policy.ListPolicyRows(self.app, self.namespace)

        parsed_args = self.check_parser(cmd, arglist, verifylist)
        cmd.take_action(parsed_args)

        lister.assert_called_with(policy_name, table_name, True)


class TestSimulatePolicy(common.TestCongressBase):

    def test_simulate_policy(self):
        policy_name = 'classification'
        action_name = 'action'
        sequence = 'q(1)'
        query = 'error(x)'
        arglist = [
            policy_name, query, sequence, action_name
        ]
        verifylist = [
            ('policy', policy_name),
            ('action_policy', action_name),
            ('sequence', sequence),
            ('query', query),
            ('delta', False)
        ]
        response = {'result': ['error(1)', 'error(2)']}

        lister = mock.Mock(return_value=response)
        self.app.client_manager.congressclient.execute_policy_action = lister
        cmd = policy.SimulatePolicy(self.app, self.namespace)

        parsed_args = self.check_parser(cmd, arglist, verifylist)
        cmd.take_action(parsed_args)

        body = {'action_policy': action_name,
                'sequence': sequence,
                'query': query}
        lister.assert_called_with(policy_name=policy_name,
                                  action='simulate',
                                  trace=False,
                                  delta=False,
                                  body=body)

    def test_simulate_policy_delta(self):
        policy_name = 'classification'
        action_name = 'action'
        sequence = 'q(1)'
        query = 'error(x)'
        arglist = [
            policy_name, query, sequence, action_name, "--delta"
        ]
        verifylist = [
            ('policy', policy_name),
            ('action_policy', action_name),
            ('sequence', sequence),
            ('query', query),
            ('delta', True)
        ]
        response = {'result': ['error(1)', 'error(2)']}

        lister = mock.Mock(return_value=response)
        self.app.client_manager.congressclient.execute_policy_action = lister
        cmd = policy.SimulatePolicy(self.app, self.namespace)

        parsed_args = self.check_parser(cmd, arglist, verifylist)
        cmd.take_action(parsed_args)

        body = {'action_policy': action_name,
                'sequence': sequence,
                'query': query}
        lister.assert_called_with(policy_name=policy_name,
                                  action='simulate',
                                  trace=False,
                                  delta=True,
                                  body=body)

    def test_simulate_policy_trace(self):
        policy_name = 'classification'
        action_name = 'action'
        sequence = 'q(1)'
        query = 'error(x)'
        arglist = [
            policy_name, query, sequence, action_name, "--trace"
        ]
        verifylist = [
            ('policy', policy_name),
            ('action_policy', action_name),
            ('sequence', sequence),
            ('query', query),
            ('trace', True)
        ]
        response = {'result': ['error(1)', 'error(2)'], 'trace': 'Call'}

        lister = mock.Mock(return_value=response)
        self.app.client_manager.congressclient.execute_policy_action = lister
        cmd = policy.SimulatePolicy(self.app, self.namespace)

        parsed_args = self.check_parser(cmd, arglist, verifylist)
        cmd.take_action(parsed_args)

        body = {'action_policy': action_name,
                'sequence': sequence,
                'query': query}
        lister.assert_called_with(policy_name=policy_name,
                                  action='simulate',
                                  trace=True,
                                  delta=False,
                                  body=body)


class TestGet(common.TestCongressBase):

    def test_create_policy_rule(self):
        policy_name = 'classification'
        rule = "p(x) :- q(x)"
        id = "e531f2b3-3d97-42c0-b3b5-b7b6ab532018"
        response = {"comment": "None",
                    "id": id,
                    "rule": rule}

        arglist = [policy_name, id]
        verifylist = [
            ('policy_name', policy_name),
            ('rule_id', id),
        ]

        mocker = mock.Mock(return_value=response)
        self.app.client_manager.congressclient.show_policy_rule = mocker
        self.app.client_manager.congressclient.list_policy_rules = mock.Mock()
        cmd = policy.ShowPolicyRule(self.app, self.namespace)
        parsed_args = self.check_parser(cmd, arglist, verifylist)
        with mock.patch.object(utils, "get_resource_id_from_name",
                               return_value="id"):
            result = list(cmd.take_action(parsed_args))
        filtered = [('comment', 'id', 'rule'),
                    ('None', id, rule)]
        self.assertEqual(filtered, result)



#   Copyright 2012-2013 OpenStack, LLC.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

"""Datasource action implemenations"""

from cliff import command
from cliff import lister
from cliff import show
from oslo_log import log as logging
from oslo_serialization import jsonutils
import six

from congressclient.common import parseractions
from congressclient.common import utils


class ListDatasources(lister.Lister):
    """List Datasources."""

    log = logging.getLogger(__name__ + '.ListDatasources')

    def get_parser(self, prog_name):
        parser = super(ListDatasources, self).get_parser(prog_name)
        return parser

    def take_action(self, parsed_args):
        # set default max-width
        if parsed_args.max_width == 0:
            parsed_args.max_width = 80
        client = self.app.client_manager.congressclient
        data = client.list_datasources()['results']
        # Type is always None, so disabling it for now.
        columns = ['id', 'name', 'enabled', 'driver', 'config']
        formatters = {'config': utils.format_dict}
        return (columns,
                (utils.get_dict_properties(s, columns,
                                           formatters=formatters)
                 for s in data))


class ListDatasourceTables(lister.Lister):
    """List datasource tables."""

    log = logging.getLogger(__name__ + '.ListDatasourceTables')

    def get_parser(self, prog_name):
        parser = super(ListDatasourceTables, self).get_parser(prog_name)
        parser.add_argument(
            'datasource_name',
            metavar="<datasource>",
            help="Name or ID of the datasource")
        return parser

    def take_action(self, parsed_args):
        self.log.debug('take_action(%s)' % parsed_args)
        # set default max-width
        if parsed_args.max_width == 0:
            parsed_args.max_width = 80
        client = self.app.client_manager.congressclient
        name_or_id = parsed_args.datasource_name
        data = client.list_datasource_tables(name_or_id)['results']
        columns = ['id']
        formatters = {'DatasourceTables': utils.format_list}
        return (columns,
                (utils.get_dict_properties(s, columns,
                                           formatters=formatters)
                 for s in data))


class ShowDatasourceStatus(show.ShowOne):
    """List status for datasource."""

    log = logging.getLogger(__name__ + '.ShowDatasourceStatus')

    def get_parser(self, prog_name):
        parser = super(ShowDatasourceStatus, self).get_parser(prog_name)
        parser.add_argument(
            'datasource_name',
            metavar="<datasource>",
            help="Name or ID of the datasource")
        return parser

    def take_action(self, parsed_args):
        self.log.debug('take_action(%s)' % parsed_args)
        # set default max-width
        if parsed_args.max_width == 0:
            parsed_args.max_width = 80
        client = self.app.client_manager.congressclient
        datasource_id = parsed_args.datasource_name
        data = client.list_datasource_status(datasource_id)
        return zip(*sorted(six.iteritems(data)))


class ShowDatasourceActions(lister.Lister):
    """List supported actions for datasource."""

    log = logging.getLogger(__name__ + '.ShowDatasourceActions')

    def get_parser(self, prog_name):
        parser = super(ShowDatasourceActions, self).get_parser(prog_name)
        parser.add_argument(
            'datasource_name',
            metavar="<datasource>",
            help="Name or ID of the datasource")
        return parser

    def take_action(self, parsed_args):
        self.log.debug('take_action(%s)' % parsed_args)
        # as we know output it's long, limit column length here
        if parsed_args.max_width == 0:
            parsed_args.max_width = 80

        client = self.app.client_manager.congressclient
        datasource_id = parsed_args.datasource_name
        data = client.list_datasource_actions(datasource_id)
        formatters = {'args': utils.format_long_dict_list}
        newdata = [{'action': x['name'],
                    'args': x['args'],
                    'description': x['description']}
                   for x in data['results']]
        columns = ['action', 'args', 'description']
        return (columns, (utils.get_dict_properties(s,
                                                    columns,
                                                    formatters=formatters)
                for s in newdata))


class ShowDatasourceSchema(lister.Lister):
    """Show schema for datasource."""

    log = logging.getLogger(__name__ + '.ShowDatasourceSchema')

    def get_parser(self, prog_name):
        parser = super(ShowDatasourceSchema, self).get_parser(prog_name)
        parser.add_argument(
            'datasource_name',
            metavar="<datasource>",
            help="Name or ID of the datasource")
        return parser

    def take_action(self, parsed_args):
        self.log.debug('take_action(%s)' % parsed_args)
        # set default max-width
        if parsed_args.max_width == 0:
            parsed_args.max_width = 80
        client = self.app.client_manager.congressclient
        datasource_id = parsed_args.datasource_name
        data = client.show_datasource_schema(datasource_id)
        formatters = {'columns': utils.format_long_dict_list}
        newdata = [{'table': x['table_id'],
                    'columns': x['columns']}
                   for x in data['tables']]
        columns = ['table', 'columns']
        return (columns,
                (utils.get_dict_properties(s, columns,
                                           formatters=formatters)
                 for s in newdata))


class ShowDatasourceTableSchema(lister.Lister):
    """Show schema for datasource table."""

    log = logging.getLogger(__name__ + '.ShowDatasourceTableSchema')

    def get_parser(self, prog_name):
        parser = super(ShowDatasourceTableSchema, self).get_parser(prog_name)
        parser.add_argument(
            'datasource_name',
            metavar="<datasource>",
            help="Name or ID of the datasource")
        parser.add_argument(
            'table_name',
            metavar="<table-name>",
            help="Name of the table")
        return parser

    def take_action(self, parsed_args):
        self.log.debug('take_action(%s)' % parsed_args)
        # set default max-width
        if parsed_args.max_width == 0:
            parsed_args.max_width = 80
        client = self.app.client_manager.congressclient
        datasource_id = parsed_args.datasource_name
        data = client.show_datasource_table_schema(
            datasource_id,
            parsed_args.table_name)
        columns = ['name', 'description']
        return (columns,
                (utils.get_dict_properties(s, columns)
                 for s in data['columns']))


class ListDatasourceRows(lister.Lister):
    """List datasource rows."""

    log = logging.getLogger(__name__ + '.ListDatasourceRows')

    def get_parser(self, prog_name):
        parser = super(ListDatasourceRows, self).get_parser(prog_name)
        parser.add_argument(
            'datasource_name',
            metavar="<datasource>",
            help="Name or ID of the datasource to show")
        parser.add_argument(
            'table',
            metavar="<table>",
            help="Table to get the datasource rows from")
        return parser

    def take_action(self, parsed_args):
        self.log.debug('take_action(%s)' % parsed_args)
        # set default max-width
        if parsed_args.max_width == 0:
            parsed_args.max_width = 80
        client = self.app.client_manager.congressclient
        datasource_id = parsed_args.datasource_name
        results = client.list_datasource_rows(datasource_id,
                                              parsed_args.table)['results']
        if results:
            columns = client.show_datasource_table_schema(
                datasource_id, parsed_args.table)['columns']
            columns = [col['name'] for col in columns]
        else:
            columns = ['data']  # doesn't matter because the rows are empty
        return (columns, (x['data'] for x in results))


class ShowDatasourceTable(show.ShowOne):
    """Show Datasource Table properties."""

    log = logging.getLogger(__name__ + '.ShowDatasourceTable')

    def get_parser(self, prog_name):
        parser = super(ShowDatasourceTable, self).get_parser(prog_name)
        parser.add_argument(
            'datasource_name',
            metavar='<datasource>',
            help="Name or ID of datasource")
        parser.add_argument(
            'table_id',
            metavar='<table-id>',
            help="Table id")

        return parser

    def take_action(self, parsed_args):
        self.log.debug('take_action(%s)' % parsed_args)
        # set default max-width
        if parsed_args.max_width == 0:
            parsed_args.max_width = 80
        client = self.app.client_manager.congressclient
        data = client.show_datasource_table(parsed_args.datasource_name,
                                            parsed_args.table_id)
        return zip(*sorted(six.iteritems(data)))


class CreateDatasource(show.ShowOne):
    """Create a datasource."""

    log = logging.getLogger(__name__ + '.CreateDatasource')

    def get_parser(self, prog_name):
        parser = super(CreateDatasource, self).get_parser(prog_name)
        parser.add_argument(
            'driver',
            metavar="<datasource-driver>",
            help="Selected datasource driver")
        parser.add_argument(
            'name',
            metavar="<datasource-name>",
            help="Name you want to call the datasource")
        parser.add_argument(
            '--description',
            metavar="<datasource-description>",
            help="Description of the datasource")

        parser.add_argument(
            '--config',
            metavar="<key=value>",
            action=parseractions.KeyValueAction,
            help="config dictionary to pass in")

        return parser

    def take_action(self, parsed_args):
        self.log.debug('take_action(%s)' % parsed_args)
        # set default max-width
        if parsed_args.max_width == 0:
            parsed_args.max_width = 80
        client = self.app.client_manager.congressclient
        body = {'name': parsed_args.name,
                'driver': parsed_args.driver,
                'config': parsed_args.config}
        if parsed_args.description:
            body['description'] = parsed_args.description
        results = client.create_datasource(body)
        return zip(*sorted(six.iteritems(results)))


class DeleteDatasource(command.Command):
    """Delete a datasource."""

    log = logging.getLogger(__name__ + '.DeleteDatasource')

    def get_parser(self, prog_name):
        parser = super(DeleteDatasource, self).get_parser(prog_name)
        parser.add_argument(
            'datasource',
            metavar="<datasource>",
            help="Name or ID of the datasource to delete")
        return parser

    def take_action(self, parsed_args):
        self.log.debug('take_action(%s)' % parsed_args)
        client = self.app.client_manager.congressclient
        try:
            datasource_id = parsed_args.datasource
            client.delete_datasource(datasource_id)
        except Exception:
            # for backwards compatibility with pre-Ocata congress server,
            # try old method of explicit conversion from name to UUID
            results = client.list_datasources()
            datasource_id = utils.get_resource_id_from_name(
                parsed_args.datasource, results)
            client.delete_datasource(datasource_id)


class UpdateDatasourceRow(command.Command):
    """Update rows to a datasource table."""

    log = logging.getLogger(__name__ + '.UpdateDatasourceRow')

    def get_parser(self, prog_name):
        parser = super(UpdateDatasourceRow, self).get_parser(prog_name)
        parser.add_argument(
            'datasource',
            metavar="<datasource>",
            help="Name or ID of the datasource to Update")
        parser.add_argument(
            'table',
            metavar="<table>",
            help="Name or ID of the table to Update")
        parser.add_argument(
            'rows',
            type=jsonutils.loads,
            metavar="<rows>",
            help=("List of Rows should be formmated json style."
                  " ex. [[row1], [row2]]"))
        return parser

    def take_action(self, parsed_args):
        self.log.debug('take_action(%s)' % parsed_args)
        client = self.app.client_manager.congressclient
        body = parsed_args.rows
        client.update_datasource_rows(
            parsed_args.datasource, parsed_args.table, body)


class DatasourceRequestRefresh(command.Command):
    """Trigger a datasource to poll."""

    log = logging.getLogger(__name__ + '.DatasourceRequestRefresh')

    def get_parser(self, prog_name):
        parser = super(DatasourceRequestRefresh, self).get_parser(prog_name)
        parser.add_argument(
            'datasource',
            metavar="<datasource>",
            help="Name or ID of the datasource to poll")
        return parser

    def take_action(self, parsed_args):
        self.log.debug('take_action(%s)' % parsed_args)
        client = self.app.client_manager.congressclient
        results = client.list_datasources()
        datasource_id = utils.get_resource_id_from_name(
            parsed_args.datasource, results)
        client.request_refresh(datasource_id, {})



#   Copyright 2012-2013 OpenStack, LLC.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

"""Driver action implemenations"""

from cliff import lister
from cliff import show
from oslo_log import log as logging
import six

from congressclient.common import utils


class ListDrivers(lister.Lister):
    """List drivers."""

    log = logging.getLogger(__name__ + '.ListDrivers')

    def get_parser(self, prog_name):
        parser = super(ListDrivers, self).get_parser(prog_name)
        return parser

    def take_action(self, parsed_args):
        # set default max-width
        if parsed_args.max_width == 0:
            parsed_args.max_width = 80
        client = self.app.client_manager.congressclient
        data = client.list_drivers()['results']
        columns = ['id', 'description']
        formatters = {'Drivers': utils.format_list}
        return (columns,
                (utils.get_dict_properties(s, columns,
                                           formatters=formatters)
                 for s in data))


class ShowDriverConfig(show.ShowOne):
    """List driver tables."""

    log = logging.getLogger(__name__ + '.ShowDriverConfig')

    def get_parser(self, prog_name):
        parser = super(ShowDriverConfig, self).get_parser(prog_name)
        parser.add_argument(
            'driver',
            metavar="<datasource-driver>",
            help="Name of the datasource driver")
        return parser

    def take_action(self, parsed_args):
        self.log.debug('take_action(%s)' % parsed_args)
        # set default max-width
        if parsed_args.max_width == 0:
            parsed_args.max_width = 80
        client = self.app.client_manager.congressclient
        data = client.show_driver(
            parsed_args.driver)
        # remove table schema info from displaying
        del data['tables']
        return zip(*sorted(six.iteritems(data)))
        columns = ['id']
        formatters = {'DriverTables': utils.format_list}
        return (columns,
                (utils.get_dict_properties(s, columns,
                                           formatters=formatters)
                 for s in data))


class ShowDriverSchema(lister.Lister):
    """List datasource tables."""

    log = logging.getLogger(__name__ + '.ShowDriverSchema')

    def get_parser(self, prog_name):
        parser = super(ShowDriverSchema, self).get_parser(prog_name)
        parser.add_argument(
            'driver',
            metavar="<datasource-driver>",
            help="Name of the datasource driver")
        return parser

    def take_action(self, parsed_args):
        self.log.debug('take_action(%s)' % parsed_args)
        # set default max-width
        if parsed_args.max_width == 0:
            parsed_args.max_width = 80
        client = self.app.client_manager.congressclient
        data = client.show_driver(
            parsed_args.driver)
        formatters = {'columns': utils.format_long_dict_list}
        newdata = [{'table': x['table_id'],
                    'columns': x['columns']}
                   for x in data['tables']]
        columns = ['table', 'columns']
        return (columns,
                (utils.get_dict_properties(s, columns,
                                           formatters=formatters)
                 for s in newdata))



#   Copyright 2012-2013 OpenStack, LLC.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

import importlib
import os

from congressclient import exceptions


def env(*vars, **kwargs):
    """Search for the first defined of possibly many env vars

    Returns the first environment variable defined in vars, or
    returns the default defined in kwargs.
    """
    for v in vars:
        value = os.environ.get(v, None)
        if value:
            return value
    return kwargs.get('default', '')


def import_class(import_str):
    """Returns a class from a string including module and class

    :param import_str: a string representation of the class name
    :rtype: the requested class
    """
    mod_str, _sep, class_str = import_str.rpartition('.')
    mod = importlib.import_module(mod_str)
    return getattr(mod, class_str)


def get_client_class(api_name, version, version_map):
    """Returns the client class for the requested API version

    :param api_name: the name of the API, e.g. 'compute', 'image', etc
    :param version: the requested API version
    :param version_map: a dict of client classes keyed by version
    :rtype: a client class for the requested API version
    """
    try:
        client_path = version_map[str(version)]
    except (KeyError, ValueError):
        msg = "Invalid %s client version '%s'. must be one of: %s" % (
              (api_name, version, ', '.join(version_map.keys())))
        raise exceptions.UnsupportedVersion(msg)

    return import_class(client_path)


def format_long_dict_list(data):
    """Return a formatted string.

    :param data: a list of dicts
    :rtype: a string formatted to {a:b, c:d}, {e:f, g:h}
    """
    newdata = [str({str(key): str(value) for key, value in d.iteritems()})
               for d in data]
    return ',\n'.join(newdata) + '\n'


def format_dict(data):
    """Return a formatted string.

    :param data: a dict
    :rtype: a string formatted to {a:b, c:d}
    """
    if not isinstance(data, dict):
        return str(data)
    return str({str(key): str(value) for key, value in data.items()})


def format_list(data):
    """Return a formatted strings

    :param data: a list of strings
    :rtype: a string formatted to a,b,c
    """

    return ', '.join(data)


def get_dict_properties(item, fields, mixed_case_fields=[], formatters={}):
    """Return a tuple containing the item properties.

    :param item: a single dict resource
    :param fields: tuple of strings with the desired field names
    :param mixed_case_fields: tuple of field names to preserve case
    :param formatters: dictionary mapping field names to callables
       to format the values
    """
    row = []
    for field in fields:
        if field in mixed_case_fields:
            field_name = field.replace(' ', '_')
        else:
            field_name = field.lower().replace(' ', '_')
        data = item[field_name] if field_name in item else ''
        if field in formatters:
            row.append(formatters[field](data))
        else:
            row.append(data)
    return tuple(row)


def get_resource_id_from_name(name, results):
    # FIXME(arosen): move to common lib and add tests...
    name_match = None
    id_match = None
    double_name_match = False
    for result in results['results']:
        if result['id'] == name:
            id_match = result['id']
        if result['name'] == name:
            if name_match:
                double_name_match = True
            name_match = result['id']
    if not double_name_match and name_match:
        return name_match
    if double_name_match and not id_match:
        # NOTE(arosen): this should only occur is using congress
        # as admin and multiple projects use the same datsource name.
        raise exceptions.Conflict(
            "Multiple resources have this name %s. Please specify id." % name)
    if id_match:
        return id_match

    raise exceptions.NotFound("Resource %s not found" % name)



#   Copyright 2014 VMWare.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

"""OpenStackClient plugin for Governance service."""

from oslo_log import log as logging

from congressclient.common import utils

LOG = logging.getLogger(__name__)

DEFAULT_POLICY_API_VERSION = '1'
API_VERSION_OPTION = 'os_policy_api_version'
API_NAME = 'congressclient'
API_VERSIONS = {
    '1': 'congressclient.v1.client.Client',
}


def make_client(instance):
    """Returns a congress service client."""
    congress_client = utils.get_client_class(
        API_NAME,
        instance._api_version[API_NAME],
        API_VERSIONS)
    LOG.debug('instantiating congress client: %s', congress_client)
    return congress_client(session=instance.session,
                           auth=None,
                           interface='publicURL',
                           service_type='policy',
                           region_name=instance._region_name)


def build_option_parser(parser):
    """Hook to add global options."""
    parser.add_argument(
        '--os-policy-api-version',
        metavar='<policy-api-version>',
        default=utils.env(
            'OS_POLICY_API_VERSION',
            default=DEFAULT_POLICY_API_VERSION),
        help=('Policy API version, default=%s (Env: OS_POLICY_API_VERSION)' %
              DEFAULT_POLICY_API_VERSION))
    return parser



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import mock
from oslo_serialization import jsonutils

from congressclient.common import utils
from congressclient.osc.v1 import datasource
from congressclient.tests import common


class TestListDatasources(common.TestCongressBase):
    def test_list_datasource(self):
        datasource_name = 'neutron'
        arglist = [
        ]
        verifylist = [
        ]
        response = {
            "results": [{"id": datasource_name,
                         "name": "my_name",
                         "enabled": "True",
                         "driver": "driver1",
                         "config": None}]
        }
        lister = mock.Mock(return_value=response)
        self.app.client_manager.congressclient.list_datasources = lister
        cmd = datasource.ListDatasources(self.app, self.namespace)

        parsed_args = self.check_parser(cmd, arglist, verifylist)
        result = cmd.take_action(parsed_args)

        lister.assert_called_with()
        self.assertEqual(['id', 'name', 'enabled', 'driver', 'config'],
                         result[0])
        for row in result[1]:
            self.assertEqual(datasource_name, row[0])
            self.assertEqual("my_name", row[1])
            self.assertEqual("True", row[2])
            self.assertEqual("driver1", row[3])
            self.assertEqual("None", row[4])

    def test_list_datasource_output_not_unicode(self):
        # response json string is converted to dict by oslo jsonutils.loads(),
        # so the key and value in the dict should be unicode type.
        response = {
            u"results": [{u"id": u"neutron",
                          u"name": u"my_name",
                          u"enabled": True,
                          u"driver": 'driver1',
                          u"config": {
                              u'username': u'admin',
                              u'project_name': u'admin',
                              u'poll_time': u'10',
                              u'password': u'<hidden>',
                              u'auth_url': u'http://127.0.0.1:5000/v2.0'
                          }}]
        }
        lister = mock.Mock(return_value=response)
        self.app.client_manager.congressclient.list_datasources = lister
        cmd = datasource.ListDatasources(self.app, self.namespace)

        parsed_args = self.check_parser(cmd, [], [])
        result = cmd.take_action(parsed_args)

        lister.assert_called_with()
        self.assertEqual(['id', 'name', 'enabled', 'driver', 'config'],
                         result[0])
        # get 'config' column
        config = list(result[1])[0][-1]
        self.assertIn("'username': 'admin'", config)
        self.assertNotIn("u'username': u'admin'", config)


class TestListDatasourceTables(common.TestCongressBase):
    def test_list_datasource_tables(self):
        datasource_name = 'neutron'
        arglist = [
            datasource_name
        ]
        verifylist = [
            ('datasource_name', datasource_name)
        ]
        response = {
            "results": [{"id": "ports"},
                        {"id": "networks"}]
        }
        lister = mock.Mock(return_value=response)
        self.app.client_manager.congressclient.list_datasource_tables = lister
        cmd = datasource.ListDatasourceTables(self.app, self.namespace)

        parsed_args = self.check_parser(cmd, arglist, verifylist)

        result = cmd.take_action(parsed_args)
        lister.assert_called_with(datasource_name)
        self.assertEqual(['id'], result[0])


class TestListDatasourceStatus(common.TestCongressBase):
    def test_list_datasource_status(self):
        datasource_name = 'neutron'
        arglist = [
            datasource_name
        ]
        verifylist = [
            ('datasource_name', datasource_name)
        ]
        response = {'last_updated': "now",
                    'last_error': "None"}
        lister = mock.Mock(return_value=response)
        self.app.client_manager.congressclient.list_datasource_status = lister
        cmd = datasource.ShowDatasourceStatus(self.app, self.namespace)

        parsed_args = self.check_parser(cmd, arglist, verifylist)
        result = list(cmd.take_action(parsed_args))

        lister.assert_called_with(datasource_name)
        self.assertEqual([('last_error', 'last_updated'),
                          ('None', 'now')],
                         result)


class TestShowDatasourceActions(common.TestCongressBase):
    def test_show_datasource_actions(self):
        datasource_name = 'fake'
        arglist = [
            datasource_name
        ]
        verifylist = [
            ('datasource_name', datasource_name)
        ]
        response = {
            "results":
                [{'name': 'execute',
                  'args': [{"name": "name", "description": "None"},
                           {"name": "status", "description": "None"},
                           {"name": "id", "description": "None"}],
                  'description': 'execute action'}]
        }
        lister = mock.Mock(return_value=response)
        self.app.client_manager.congressclient.list_datasource_actions = lister
        cmd = datasource.ShowDatasourceActions(self.app, self.namespace)

        parsed_args = self.check_parser(cmd, arglist, verifylist)
        result = cmd.take_action(parsed_args)

        lister.assert_called_once_with(datasource_name)
        self.assertEqual(['action', 'args', 'description'], result[0])


class TestShowDatasourceSchema(common.TestCongressBase):
    def test_show_datasource_schema(self):
        datasource_name = 'neutron'
        arglist = [
            datasource_name
        ]
        verifylist = [
            ('datasource_name', datasource_name)
        ]
        response = {
            "tables":
                [{'table_id': 'ports',
                  'columns': [{"name": "name", "description": "None"},
                              {"name": "status", "description": "None"},
                              {"name": "id", "description": "None"}]},
                 {'table_id': 'routers',
                  'columns': [{"name": "name", "description": "None"},
                              {"name": "floating_ip", "description": "None"},
                              {"name": "id", "description": "None"}]}]
        }
        lister = mock.Mock(return_value=response)
        self.app.client_manager.congressclient.show_datasource_schema = lister
        cmd = datasource.ShowDatasourceSchema(self.app, self.namespace)

        parsed_args = self.check_parser(cmd, arglist, verifylist)
        result = cmd.take_action(parsed_args)

        lister.assert_called_with(datasource_name)
        self.assertEqual(['table', 'columns'], result[0])


class TestShowDatasourceTableSchema(common.TestCongressBase):
    def test_show_datasource_table_schema(self):
        datasource_name = 'neutron'
        table_name = 'ports'
        arglist = [
            datasource_name, table_name
        ]
        verifylist = [
            ('datasource_name', datasource_name),
            ('table_name', table_name)
        ]
        response = {
            'table_id': 'ports',
            'columns': [{"name": "name", "description": "None"},
                        {"name": "status", "description": "None"},
                        {"name": "id", "description": "None"}]
        }
        lister = mock.Mock(return_value=response)
        client = self.app.client_manager.congressclient
        client.show_datasource_table_schema = lister
        cmd = datasource.ShowDatasourceTableSchema(self.app, self.namespace)

        parsed_args = self.check_parser(cmd, arglist, verifylist)
        result = cmd.take_action(parsed_args)

        lister.assert_called_with(datasource_name, table_name)
        self.assertEqual(['name', 'description'], result[0])


class TestListDatasourceRows(common.TestCongressBase):

    def test_list_datasource_row(self):
        datasource_name = 'neutron'
        table_name = 'ports'
        arglist = [
            datasource_name, table_name
        ]
        verifylist = [
            ('datasource_name', datasource_name),
            ('table', table_name)
        ]
        response = {
            "results": [{"data": ["69abc88b-c950-4625-801b-542e84381509",
                                  "default"]}]
        }
        schema_response = {
            'table_id': 'ports',
            'columns': [{"name": "ID", "description": "None"},
                        {"name": "name", "description": "None"}]
        }

        client = self.app.client_manager.congressclient
        lister = mock.Mock(return_value=response)
        client.list_datasource_rows = lister
        schema_lister = mock.Mock(return_value=schema_response)
        client.show_datasource_table_schema = schema_lister
        cmd = datasource.ListDatasourceRows(self.app, self.namespace)

        parsed_args = self.check_parser(cmd, arglist, verifylist)
        result = cmd.take_action(parsed_args)

        lister.assert_called_with(datasource_name, table_name)
        self.assertEqual(['ID', 'name'], result[0])


class TestShowDatasourceTable(common.TestCongressBase):
    def test_show_datasource_table(self):
        datasource_name = 'neutron'
        table_id = 'ports'
        arglist = [
            datasource_name, table_id
        ]
        verifylist = [
            ('datasource_name', datasource_name),
            ('table_id', table_id)
        ]
        response = {
            'id': 'ports',
        }
        lister = mock.Mock(return_value=response)
        client = self.app.client_manager.congressclient
        client.show_datasource_table = lister
        cmd = datasource.ShowDatasourceTable(self.app, self.namespace)
        expected_ret = [('id',), ('ports',)]

        parsed_args = self.check_parser(cmd, arglist, verifylist)
        result = list(cmd.take_action(parsed_args))

        self.assertEqual(expected_ret, result)


class TestCreateDatasource(common.TestCongressBase):

    def test_create_datasource(self):
        driver = 'neutronv2'
        name = 'arosen-neutronv2'
        response = {"description": '',
                    "config": {"username": "admin",
                               "project_name": "admin",
                               "password": "password",
                               "auth_url": "http://127.0.0.1:5000/v2.0"},
                    "enabled": True,
                    "owner": "user",
                    "driver": "neutronv2",
                    "type": None,
                    "id": "b72f81a0-32b5-4bf4-a1f6-d69c09c42cec",
                    "name": "arosen-neutronv2"}

        arglist = [driver, name,
                   "--config", "username=admin",
                   "--config", "password=password",
                   "--config", "auth_url=http://1.1.1.1/foo",
                   "--config", "project_name=admin"]
        verifylist = [
            ('driver', driver),
            ('name', name),
            ('config', {'username': 'admin', 'password': 'password',
                        'auth_url': 'http://1.1.1.1/foo',
                        'project_name': 'admin'}),
        ]

        mocker = mock.Mock(return_value=response)
        self.app.client_manager.congressclient.create_datasource = mocker
        cmd = datasource.CreateDatasource(self.app, self.namespace)
        parsed_args = self.check_parser(cmd, arglist, verifylist)
        result = list(cmd.take_action(parsed_args))
        filtered = [('config', 'description',
                     'driver', 'enabled', 'id', 'name',
                     'owner', 'type'),
                    (response['config'], response['description'],
                     response['driver'], response['enabled'],
                     response['id'], response['name'],
                     response['owner'], response['type'])]
        self.assertEqual(filtered, result)


class TestDeleteDatasourceDriver(common.TestCongressBase):

    def test_delete_datasource(self):
        driver = 'neutronv2'

        arglist = [driver]
        verifylist = [('datasource', driver), ]

        mocker = mock.Mock(return_value=None)
        self.app.client_manager.congressclient.delete_datasource = mocker
        cmd = datasource.DeleteDatasource(self.app, self.namespace)
        parsed_args = self.check_parser(cmd, arglist, verifylist)
        result = cmd.take_action(parsed_args)
        mocker.assert_called_with(driver)
        self.assertIsNone(result)


class TestUpdateDatasourceRow(common.TestCongressBase):

    def test_update_datasource_row(self):
        driver = 'push'
        table_name = 'table'
        rows = [["data1", "data2"],
                ["data3", "data4"]]

        arglist = [driver, table_name, jsonutils.dumps(rows)]
        verifylist = [('datasource', driver),
                      ('table', table_name),
                      ('rows', rows)]

        mocker = mock.Mock(return_value=None)
        self.app.client_manager.congressclient.update_datasource_rows = mocker
        self.app.client_manager.congressclient.list_datasources = mock.Mock()

        cmd = datasource.UpdateDatasourceRow(self.app, self.namespace)
        parsed_args = self.check_parser(cmd, arglist, verifylist)
        with mock.patch.object(utils, 'get_resource_id_from_name',
                               return_value="push"):
            cmd.take_action(parsed_args)
        mocker.assert_called_with(driver, table_name, rows)


class TestDatasourceRequestRefresh(common.TestCongressBase):

    def test_datasource_request_refresh(self):
        driver = 'neutronv2'

        arglist = [driver]
        verifylist = [('datasource', driver), ]

        mocker = mock.Mock(return_value=None)
        self.app.client_manager.congressclient.request_refresh = mocker
        self.app.client_manager.congressclient.list_datasources = mock.Mock()
        cmd = datasource.DatasourceRequestRefresh(self.app, self.namespace)
        parsed_args = self.check_parser(cmd, arglist, verifylist)
        with mock.patch.object(utils, "get_resource_id_from_name",
                               return_value="id"):
            result = cmd.take_action(parsed_args)
        mocker.assert_called_with("id", {})
        self.assertIsNone(result)



#   Copyright 2013 OpenStack Foundation
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

"""argparse Custom Actions"""

import argparse


class KeyValueAction(argparse.Action):
    """A custom action to parse arguments as key=value pairs

    Ensures that ``dest`` is a dict
    """

    def __call__(self, parser, namespace, values, option_string=None):
        # Make sure we have an empty dict rather than None
        if getattr(namespace, self.dest, None) is None:
            setattr(namespace, self.dest, {})

        # Add value if an assignment else remove it
        if '=' in values:
            getattr(namespace, self.dest, {}).update([values.split('=', 1)])
        else:
            getattr(namespace, self.dest, {}).pop(values, None)


class RangeAction(argparse.Action):
    """A custom action to parse a single value or a range of values

    Parses single integer values or a range of integer values delimited
    by a colon and returns a tuple of integers:
    '4' sets ``dest`` to (4, 4)
    '6:9' sets ``dest`` to (6, 9)
    """

    def __call__(self, parser, namespace, values, option_string=None):
        range = values.split(':')
        if len(range) == 0:
            # Nothing passed, return a zero default
            setattr(namespace, self.dest, (0, 0))
        elif len(range) == 1:
            # Only a single value is present
            setattr(namespace, self.dest, (int(range[0]), int(range[0])))
        elif len(range) == 2:
            # Range of two values
            if int(range[0]) <= int(range[1]):
                setattr(namespace, self.dest, (int(range[0]), int(range[1])))
            else:
                msg = ("Invalid range, %s is not less than %s" %
                       (range[0], range[1]))
                raise argparse.ArgumentError(self, msg)
        else:
            # Too many values
            msg = "Invalid range, too many values"
            raise argparse.ArgumentError(self, msg)
